"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var remote_1 = require("./remote");
var constant_1 = require("./constant");
var breakingChanges = [];
exports.makeHistory = function (commits, useGroup) {
    if (useGroup === void 0) { useGroup = false; }
    var history = useGroup ? makeGroupHistory(commits) : makePlainHistory(commits);
    if (breakingChanges.length) {
        history += "\n\n### BREAKING CHANGES";
        breakingChanges.forEach(function (breakingChange) {
            history += "\n- " + breakingChange;
        });
    }
    return history;
};
var makeGroupHistory = function (commits) {
    var groupCommits = [];
    var _loop_1 = function (group) {
        var targetGroup = commits.filter(function (commit) {
            return new RegExp(constant_1.GROUP_MAP[group], 'i').test(commit.type);
        });
        if (targetGroup.length) {
            groupCommits.push("\n### " + group);
            targetGroup.forEach(function (commit) {
                extractLabels(commit);
                var hash = commit.hash, shortHash = commit.shortHash, pureSubject = commit.pureSubject, breakingChange = commit.breakingChange;
                if (breakingChange) {
                    breakingChanges.push(breakingChange);
                }
                groupCommits.push("- " + pureSubject + " [`" + shortHash + "`](" + remote_1.getRemoteLink().commitLink + "/" + hash + ")");
            });
        }
    };
    for (var group in constant_1.GROUP_MAP) {
        _loop_1(group);
    }
    return groupCommits.join('\n');
};
var makePlainHistory = function (commits) {
    var plainHistory = '';
    commits.forEach(function (commit) {
        extractLabels(commit);
        var hash = commit.hash, shortHash = commit.shortHash, subject = commit.subject, breakingChange = commit.breakingChange;
        if (breakingChange) {
            breakingChanges.push(breakingChange);
        }
        plainHistory += "\n- " + subject + " [`" + shortHash + "`](" + remote_1.getRemoteLink().commitLink + "/" + hash + ")";
    });
    return plainHistory;
};
var extractLabels = function (commit) {
    var body = commit.body;
    var reg = /(?:(?<![/\w-.])\w[\w-.]+\/\w[\w-.]+|\B)[#|\!][1-9]\d*\b/g;
    var labels = body.match(reg);
    if (labels !== null) {
        commit.subject += labels.join(' ');
        commit.pureSubject += labels.join(' ');
    }
};
