#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var fs_1 = require("fs");
var perf_hooks_1 = require("perf_hooks");
var commander_1 = require("commander");
var log_1 = require("./log");
var constant_1 = require("./constant");
var libPath = path_1.resolve(__dirname, '../');
var startTime = perf_hooks_1.performance.now();
var program = new commander_1.Command();
program
    .version(require(libPath + "/package.json").version, '-v, --version')
    .option('-o, --output <file>', 'output file', constant_1.DEFAULT_OPTIONS.OUTPUT)
    .option('-l, --commit-limit <count>', 'number of commits to display per release', constant_1.DEFAULT_OPTIONS.COMMIT_LIMIT)
    .option('-c, --version-limit <count>', 'number of version to release', constant_1.DEFAULT_OPTIONS.VERSION_LIMIT)
    .option('--no-date', 'remove date')
    .option('--no-group', 'Do not group the commit history base on Conventional Commits for commit guidelines')
    .option('--filter <pattern>', 'filter commit by given reg pattern, for example: "^(new|fix(e(d|s))?)$"')
    .parse(process.argv);
var logContent = log_1.generateBaseTags({
    showDate: program.date,
    useGroup: program.group,
    useFilter: program.filter,
    commitLimit: program.commitLimit,
    versionLimit: program.versionLimit
});
fs_1.writeFileSync(process.cwd() + "/" + program.output, logContent);
var timeSpent = perf_hooks_1.performance.now() - startTime;
console.log("\u2728  ChangeLog was generated in " + (timeSpent / 1000).toFixed(3) + "s.");
