"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var semver_compare_1 = __importDefault(require("semver-compare"));
var commit_1 = require("./commit");
var history_1 = require("./history");
var remote_1 = require("./remote");
var utils_1 = require("./utils");
var constant_1 = require("./constant");
var rootDir = process.cwd();
var gitDir = rootDir + "/.git";
var gitTagDir = gitDir + "/refs/tags";
var gitTags;
var versions = [];
var logs = [];
var compare = function (tag1, tag2) {
    tag1 = utils_1.cleanTag(tag1);
    tag2 = utils_1.cleanTag(tag2);
    if (!tag1 || !tag2) {
        return 0;
    }
    return semver_compare_1.default(tag1, tag2);
};
exports.generateBaseTags = function (options) {
    var e_1, _a;
    if (!utils_1.fsExistsSync(gitTagDir)) {
        throw constant_1.ERR_MSG.NO_GIT;
    }
    if (utils_1.fsExistsSync(gitTagDir)) {
        gitTags = fs_1.readdirSync(gitTagDir);
        if (!gitTags.length) {
            throw constant_1.ERR_MSG.NO_TAG;
        }
    }
    else {
        throw constant_1.ERR_MSG.NO_TAG;
    }
    gitTags = __spread(new Set(gitTags));
    gitTags = gitTags.sort(compare).reverse();
    try {
        var firstTag = gitTags[gitTags.length - 1];
        var firstTagIsoDate = utils_1.getExecResult("git log -1 --pretty=\"%ci\" " + firstTag);
        gitTags.reduce(function (prev, next) {
            var isoDate = utils_1.getExecResult("git log -1 --pretty=\"%ci\" " + prev);
            versions.push({
                name: prev,
                diff: next + "..." + prev,
                date: isoDate.split(' ')[0] || ''
            });
            return next;
        });
        versions.push({
            name: firstTag,
            diff: firstTag,
            date: firstTagIsoDate.split(' ')[0] || ''
        });
        if (options.versionLimit && versions.length > options.versionLimit) {
            versions = versions.slice(0, options.versionLimit);
        }
        versions = versions.map(function (version) {
            return __assign(__assign({}, version), { commits: commit_1.parseCommit(version) });
        });
        logs.push('# ChangeLog');
        logs.push('\nThis document was generated by [autochangelog](https://github.com/roshanca/autochangelog) automatically.');
        var _loop_1 = function (version) {
            var name_1 = version.name, date = version.date, diff = version.diff, _a = version.commits, commits = _a === void 0 ? [] : _a;
            var isInitialVersion = !diff.includes('...');
            var nameUrl = isInitialVersion
                ? remote_1.getRemoteUrl() + '/commits/' + diff
                : remote_1.getRemoteLink().compareLink + '/' + diff;
            logs.push("\n## [" + name_1 + "](" + nameUrl + ") " + (options.showDate ? '(' + date + ')' : ''));
            if (options.commitLimit && commits.length > options.commitLimit) {
                commits = commits.slice(0, options.commitLimit);
            }
            if (options.useFilter) {
                var reg_1 = new RegExp(options.useFilter, 'i');
                commits = commits.filter(function (commit) {
                    return reg_1.test(commit.type);
                });
            }
            var history_2 = history_1.makeHistory(commits, options.useGroup);
            logs.push(history_2);
        };
        try {
            for (var versions_1 = __values(versions), versions_1_1 = versions_1.next(); !versions_1_1.done; versions_1_1 = versions_1.next()) {
                var version = versions_1_1.value;
                _loop_1(version);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (versions_1_1 && !versions_1_1.done && (_a = versions_1.return)) _a.call(versions_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var logContent = logs.join('\n');
        logContent = logContent.replace(/(#([1-9]\d*))/g, "[`$1`](" + remote_1.getRemoteLink().issueLink + "/$2)");
        logContent = logContent.replace(/(\!([1-9]\d*))/g, "[`$1`](" + remote_1.getRemoteLink().mergeLink + "/$2)");
        return logContent;
    }
    catch (err) {
        throw err;
    }
};
